%% 1D Blazed Grating with derivatives
%% verification driver for automatic differentiation 
keys = struct();

%% vacuum wavelengths
keys.lambda0 = 193e-9;

%% incidence angles
keys.theta = 20;
keys.phi = 30; 

%% sidewall angles, width and height of the grating
keys.alpha = 86;
keys.beta = 15;
keys.width = 200;
keys.height = 65.4;


%% differentiation order
keys.dOrder = 1;

%% solve project with automatic derivatives
%% use temporary project folder (option 'temporary'='yes')
keys.precision = 1e-2;
results = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');
fourierTransform_transmitted = results{2};

ft_s = fourierTransform_transmitted.ElectricFieldStrength{1};
ft_p = fourierTransform_transmitted.ElectricFieldStrength{2};

d_ft_s{1} =  fourierTransform_transmitted.d_Lambda0.ElectricFieldStrength{1};
d_ft_p{1} =  fourierTransform_transmitted.d_Lambda0.ElectricFieldStrength{2};

d_ft_s{2} =  fourierTransform_transmitted.d_Theta.ElectricFieldStrength{1};
d_ft_p{2} =  fourierTransform_transmitted.d_Theta.ElectricFieldStrength{2};

d_ft_s{3} =  fourierTransform_transmitted.d_Phi.ElectricFieldStrength{1};
d_ft_p{3} =  fourierTransform_transmitted.d_Phi.ElectricFieldStrength{2};

d_ft_s{4} =  fourierTransform_transmitted.d_Alpha.ElectricFieldStrength{1};
d_ft_p{4} =  fourierTransform_transmitted.d_Alpha.ElectricFieldStrength{2};

d_ft_s{5} =  fourierTransform_transmitted.d_Beta.ElectricFieldStrength{1};
d_ft_p{5} =  fourierTransform_transmitted.d_Beta.ElectricFieldStrength{2};

d_ft_s{6} =  fourierTransform_transmitted.d_Height.ElectricFieldStrength{1};
d_ft_p{6} =  fourierTransform_transmitted.d_Height.ElectricFieldStrength{2};

d_ft_s{7} =  fourierTransform_transmitted.d_Width.ElectricFieldStrength{1};
d_ft_p{7} =  fourierTransform_transmitted.d_Width.ElectricFieldStrength{2};

%% To verify the automatic differentiation compare the results to
%% to a numerical differentiation using  differential quotient
%% It is necessary to increase overall precision because
%% significant digits cancel out in the difference quotient 

% rerun primary project with higher precision
keys.precision = 1e-6;
keys.dOrder = 0;
result = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');
fourierTransform_transmitted = result{2};
ft_s_0 = fourierTransform_transmitted.ElectricFieldStrength{1};
ft_p_0 = fourierTransform_transmitted.ElectricFieldStrength{2};

delta = 1e-3;
for iPara = 1 : 7
   delta_ = delta;
   if (iPara==1) delta_ = delta*1e-9; end % lambda0 scale in nanometer

   keys_delta = keys;
   if (iPara==1)  keys_delta.lambda0 =  keys.lambda0+delta_; end
   if (iPara==2)  keys_delta.theta =  keys.theta+delta_; end
   if (iPara==3)  keys_delta.phi   =  keys.phi+delta_; end
   if (iPara==4)  keys_delta.alpha =  keys.alpha +delta_; end
   if (iPara==5)  keys_delta.beta  =  keys.beta+delta_; end
   if (iPara==6)  keys_delta.height=  keys.height+delta_; end
   if (iPara==7)  keys_delta.width =  keys.width +delta_; end          
   
   results_delta = jcmwave_solve('project.jcmp', keys_delta, 'temporary', 'yes');
   fourierTransform_transmitted_delta = results_delta{2};
   ft_s_delta = fourierTransform_transmitted_delta.ElectricFieldStrength{1};
   ft_p_delta = fourierTransform_transmitted_delta.ElectricFieldStrength{2};

   % difference quotient
   dq_ft_s{iPara} =  (ft_s_delta-ft_s_0)/delta_;
   dq_ft_p{iPara} =  (ft_p_delta-ft_p_0)/delta_;   
end


fprintf('\ncomparison automatic differentation vs. differential quotient:'); 
for iPara = 1 : 7
   err_s = max(max(abs(d_ft_s{iPara}-dq_ft_s{iPara})))/max(max(abs(d_ft_s{iPara})));
   err_p = max(max(abs(d_ft_p{iPara}-dq_ft_p{iPara})))/max(max(abs(d_ft_p{iPara})));
  fprintf('\niPara: %d, err_s: %2.4e err_p: %2.4e', iPara, err_s, err_p);
end
fprintf('\n');


