## 1D grating example
## This driver performs a scan over angle of incidence
import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import jcmwave
import numpy as np
import matplotlib.pyplot as plt

## vacuum wavelengths
lambda0 = 193e-9

## inclination angle in degree
thetas = np.linspace(-25,25,91) 

## azimuthal angle in degree
phi = 20

## perform the scan and store -1,0,+1 diffraction orders for
## incident s and p polarization
keys = {}
keys['lambda0'] = lambda0
keys['line_width']=200
diffraction_order_p = np.zeros((3,len(thetas)))
diffraction_order_s = np.zeros((3,len(thetas)))

for iRun in range(len(thetas)):
  theta = thetas[iRun]
  keys['thetaPhi'] = np.array([theta, phi])
  
  # get results
  result = jcmwave.solve('project.jcmp', keys)

  # gather transmission results
  fourierTransform = result[1]
  powerFluxes_t = jcmwave.convert2powerflux(fourierTransform)

  for iOrder in range(-1,2):
    iMode = np.equal(fourierTransform['N1'],iOrder)
    if not iMode.any():
      diffraction_order_s[iOrder+1, iRun] = float('nan')
      diffraction_order_p[iOrder+1, iRun] = float('nan')
    else:
      diffraction_order_s[iOrder+1, iRun] = powerFluxes_t['PowerFluxDensity'][0][iMode,2]
      diffraction_order_p[iOrder+1, iRun] = powerFluxes_t['PowerFluxDensity'][1][iMode,2]

# plot results
f, (ax1, ax2) = plt.subplots(2,sharex=True,figsize=(10,12))
x = np.sin(np.radians(thetas))
ax1.plot(x,diffraction_order_s[0],'.b-',label='order=-1')
ax1.plot(x,diffraction_order_s[1],'.r-',label='order=0')
ax1.plot(x,diffraction_order_s[2],'.g-',label='order=1')
ax1.set_title('S-Polarisation'); ax1.set_ylabel('Transmission')
ax1.legend(); ax1.set_xlim(-0.45,0.65); ax1.set_ylim(0.05,0.4)
ax2.plot(x,diffraction_order_p[0],'.b-',label='order=-1')
ax2.plot(x,diffraction_order_p[1],'.r-',label='order=0')
ax2.plot(x,diffraction_order_p[2],'.g-',label='order=1')
ax2.set_title('P-Polarisation'); ax2.set_ylabel('Transmission')
ax2.set_xlabel(r'Inclination [sin($\theta$)]')
ax2.legend(); ax2.set_xlim(-0.45,0.65); ax2.set_ylim(0.05,1.0)
plt.suptitle('1D Grating')
f.show()
plt.savefig('pythonPlot.png')
