%% 1D grating example
%% This driver performs a scan over the with of the line

%% vacuum wavelengths
keys.lambda0 = 193e-9;

%% line width scan sample positions 
line_widths = linspace(150,250,10);

%% fix the incidence angle
keys.thetaPhi = [20 25];

%% perform the scan and store -1,0,+1 diffraction orders for
%% incident s and p polarization (transmission and reflection)
diffraction_order_s_t = [];
diffraction_order_p_t = [];
diffraction_order_s_r = [];
diffraction_order_p_r = [];

for iRun = 1 : length(line_widths)
  keys.line_width = line_widths(iRun);

  %% solve the project 
  %% use temporary project folder (option 'temporary'='yes')
  results = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');

  %% gather transmission results
  fourierTransform_t = results{2};
  powerFluxes_t = jcmwave_convert2powerflux(fourierTransform_t);

  for iOrder=-1 : 1
    iMode = find(fourierTransform_t.N1==iOrder);
    if isempty(iMode)
      diffraction_order_s_t(iOrder+2, iRun) = nan;
      diffraction_order_p_t(iOrder+2, iRun) = nan;
    else
      diffraction_order_s_t(iOrder+2, iRun) = powerFluxes_t.PowerFluxDensity{1}(iMode, 3);
      diffraction_order_p_t(iOrder+2, iRun) = powerFluxes_t.PowerFluxDensity{2}(iMode, 3);
    end 
  end

  %% gather reflection results
  fourierTransform_r = results{3};
  powerFluxes_r = jcmwave_convert2powerflux(fourierTransform_r);

  for iOrder=-1 : 1
    iMode = find(fourierTransform_r.N1==iOrder);
    if isempty(iMode)
      diffraction_order_s_r(iOrder+2, iRun) = nan;
      diffraction_order_p_r(iOrder+2, iRun) = nan;
    else
      diffraction_order_s_r(iOrder+2, iRun) = powerFluxes_r.PowerFluxDensity{1}(iMode, 3);
      diffraction_order_p_r(iOrder+2, iRun) = powerFluxes_r.PowerFluxDensity{2}(iMode, 3);
    end 
  end
end

% plot results
fig = figure(1);
clf
set(fig, 'Units', 'normalized', 'Position', [0.2,0.1,0.4,0.8]);

x = line_widths;
subplot(2,2,1);
hold on
plot(x,diffraction_order_s_t(1,:),'-b.', 'LineWidth', 2);
plot(x,diffraction_order_s_t(2,:),'-r.', 'LineWidth', 2);
plot(x,diffraction_order_s_t(3,:),'-g.', 'LineWidth', 2);
title('S-Polarisation'); 
xlabel('line width [nm]');
ylabel('Transmission');
legend('order=-1','order=0','order=1')
set(gca, 'FontSize', 12)

subplot(2,2,3);
hold on
plot(x,diffraction_order_p_t(1,:),'-b.', 'LineWidth', 2);
plot(x,diffraction_order_p_t(2,:),'-r.', 'LineWidth', 2);
plot(x,diffraction_order_p_t(3,:),'-g.', 'LineWidth', 2);
title('P-Polarisation'); 
xlabel('line width [nm]');
ylabel('Transmission');
legend('order=-1','order=0','order=1')
set(gca, 'FontSize', 12)
hold off

subplot(2,2,2);
hold on
plot(x,diffraction_order_s_r(1,:),'-b.', 'LineWidth', 2);
plot(x,diffraction_order_s_r(2,:),'-r.', 'LineWidth', 2);
plot(x,diffraction_order_s_r(3,:),'-g.', 'LineWidth', 2);
title('S-Polarisation'); 
xlabel('line width [nm]');
ylabel('Reflection');
legend('order=-1','order=0','order=1')
set(gca, 'FontSize', 12)

subplot(2,2,4);
hold on
plot(x,diffraction_order_p_r(1,:),'-b.', 'LineWidth', 2);
plot(x,diffraction_order_p_r(2,:),'-r.', 'LineWidth', 2);
plot(x,diffraction_order_p_r(3,:),'-g.', 'LineWidth', 2);
title('P-Polarisation');
xlabel('line width [nm]');
ylabel('Reflection');
legend('order=-1','order=0','order=1')
suptitle('1D Grating, line width scan');
set(gca, 'FontSize', 12)
hold off

