%% 1D grating example
%% This simple driver directly starts the simulation

results = jcmwave_solve('project.jcmp');

fourierTransform_t = results{2};
fourierTransform_r = results{3};

%% The transmission and reflection coefficients are the
%% quotients between the power fluxes in normal direction 
%% of the transmitted (reflected) fields and of the incoming plane wave.
%% This can be computed from the Poynting vector of a plane wave, 
%% P=0.5*cross(E, conj(H)) and then taking the z-component
%% You can use the script jcmwave_convert2powerflux applied on the
%% Fourier transform output to compute the Poynting vector for each diffraction mode. 
%% The power flux density of the incoming field has been already normalized
%% (by PowerFluxScaling=UnitNormal in sources.jcm)

powerFluxes_t = jcmwave_convert2powerflux(fourierTransform_t);
powerFluxes_r = jcmwave_convert2powerflux(fourierTransform_r);

%% transmitted flux in normal direction
P_s_t = sum(powerFluxes_t.PowerFluxDensity{1}(:, 3));
P_p_t = sum(powerFluxes_t.PowerFluxDensity{2}(:, 3));

%% reflected flux in normal direction
P_s_r = sum(powerFluxes_r.PowerFluxDensity{1}(:, 3));
P_p_r = sum(powerFluxes_r.PowerFluxDensity{2}(:, 3));

fprintf('\nTotal transmission for s-polarized incoming field: T=%2.4g', sum(P_s_t));
fprintf('\nTotal reflection for s-polarized incoming field: R=%2.4g', sum(P_s_r));
fprintf('\nTotal transmission for s-polarized incoming field: T=%2.4g', sum(P_p_t));
fprintf('\nTotal reflection for s-polarized incoming field: R=%2.4g', sum(P_p_r));
fprintf('\n');
