## 1D grating example
## This simple driver directly starts the simulation
import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import jcmwave
import numpy as np
import matplotlib.pyplot as plt

results = jcmwave.solve('project.jcmp')

fourierTransform_t = results[1]
fourierTransform_r = results[2]

## The transmission and reflection coefficients are the
## quotients between the power fluxes in normal direction 
## of the transmitted (reflected) fields and of the incoming plane wave.
## This can be computed from the Poynting vector of a plane wave, 
## P=0.5*cross(E, conj(H)) and then taking the z-component
## You can use the script jcmwave_convert2powerflux applied on the
## Fourier transform output to compute the Poynting vector for each diffraction mode. 
## The power flux density of the incoming field has been already normalized
## (by PowerFluxScaling=UnitNormal in sources.jcm)

powerFluxes_t = jcmwave.convert2powerflux(fourierTransform_t)
powerFluxes_r = jcmwave.convert2powerflux(fourierTransform_r)

## transmitted flux in normal direction
P_s_t = sum(powerFluxes_t['PowerFluxDensity'][0][:,2])
P_p_t = sum(powerFluxes_t['PowerFluxDensity'][1][:,2])

## reflected flux in normal direction
P_s_r = sum(powerFluxes_r['PowerFluxDensity'][0][:,2])
P_p_r = sum(powerFluxes_r['PowerFluxDensity'][1][:,2])

print('Total transmission for s-polarized incoming field: T=%2.4g' % P_s_t)
print('Total reflection for s-polarized incoming field: R=%2.4g' % P_s_r)
print('Total transmission for s-polarized incoming field: T=%2.4g' % P_p_t)
print('Total reflection for s-polarized incoming field: R=%2.4g' % P_p_r)
