## 2D grating example (square lattice)
## This driver performs a scan over angle of incidence
import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import jcmwave
import numpy as np
import matplotlib.pyplot as plt

## vacuum wavelengths
lambda0 = 193e-9

## inclination angle in degree
thetas = np.linspace(-15,15,11) 

## azimuthal angle in degree
phi = 20

## perform the scan and store -1,0,+1 diffraction orders for
## incident s and p polarization
keys = {}
keys['lambda0'] = lambda0

## fix pattern lower/upper pattern width
keys['pattern_width_bottom'] = 200
keys['pattern_width_top'] = 190

diffraction_order_p = np.zeros((3,3,len(thetas)))
diffraction_order_s = np.zeros((3,3,len(thetas)))

for iRun in range(len(thetas)):
  theta = thetas[iRun]
  keys['thetaPhi'] = np.array([theta, phi])

  ## get results
  result = jcmwave.solve('project.jcmp', keys)
  fourierTransform = result[1]

  t_s = np.sum(abs(fourierTransform['ElectricFieldStrength'][0])**2, axis=1)
  t_p = np.sum(abs(fourierTransform['ElectricFieldStrength'][1])**2, axis=1)
  
  for iOrder in range(-1,2):
    for jOrder in range(-1,2):
      iMode = np.logical_and(np.equal(fourierTransform['N1'],iOrder),np.equal(fourierTransform['N2'],jOrder))
      if not iMode.any():
        diffraction_order_s[iOrder+1, jOrder+1, iRun] = float('nan')
        diffraction_order_p[iOrder+1, jOrder+1, iRun] = float('nan')
      else:
        diffraction_order_s[iOrder+1, jOrder+1, iRun] = t_s[iMode]
        diffraction_order_p[iOrder+1, jOrder+1, iRun] = t_p[iMode]


## plot results
x = np.sin(np.radians(thetas))
f, (ax1, ax2) = plt.subplots(2,sharex=True,figsize=(10,13))
for iOrder in range(0,3):
  for jOrder in range(0,3):
    ax1.plot(x,diffraction_order_s[iOrder,jOrder],'.-',label='order=[{0}, {1}]'.format(iOrder-1,jOrder-1))
    ax1.set_title('S-Polarisation'); ax1.set_ylabel('Transmission')
    ax1.legend(); ax1.set_xlim(-0.45,0.75);
    ax2.plot(x,diffraction_order_p[iOrder,jOrder],'.-',label='order=[{0}, {1}]'.format(iOrder-1,jOrder-1))
    ax2.set_title('P-Polarisation'); ax2.set_ylabel('Transmission')
    ax2.set_xlabel(r'Inclination [sin($\theta$)]')
    ax2.legend(); ax2.set_xlim(-0.45,0.75);
    plt.suptitle('2D Grating Square')
f.show()
