%function run_derivatives()

keys.substratePermittivity=2.25;
keys.lambda0=1.93e-7;
keys.theta=20;

 results = jcmwave_solve('project.jcmp',keys);
 t=results{3};

parameters={};
parameters{1}="LayerInterface";
parameters{2}="SubstratePermittivity";
parameters{3}="Lambda0";
parameters{4}="Theta";

iK=1;

substratePermittivity=keys.substratePermittivity;
lambda0=keys.lambda0;
theta=keys.theta;
layerInterface=0;

[ds,dp,ddelta,dpsi]=Ellipsometry(layerInterface,substratePermittivity,lambda0,theta);
[dsJCM,dpJCM,ddeltaJCM,dpsiJCM]=EllipsometryJCM(t,iK);

fprintf("\nAccuracy of primary value:\n");
fprintf('jonesSS:   %e  (reference = %s)\n',abs(ds-dsJCM),formatComplex(ds));
fprintf('jonesPP:   %e  (reference = %s)\n',abs(dp-dpJCM),formatComplex(dp));
  
for iP=1:length(parameters)
  [ds,dp,ddelta,dpsi]=dEllipsometry(layerInterface,substratePermittivity,lambda0,theta,parameters{iP});
  [dsJCM,dpJCM,ddeltaJCM,dpsiJCM]=dEllipsometryJCM(t,iK,parameters{iP});
  
  fprintf("\nAccuracy of first derivative w.r.t. \'%s\':\n",parameters{iP});
  fprintf('jonesSS:   %e  (reference = %s)\n',abs(ds-dsJCM),formatComplex(ds));
  fprintf('jonesPP:   %e  (reference = %s)\n',abs(dp-dpJCM),formatComplex(dp));
end

for iP1=1:length(parameters)
  for iP2=1:iP1
    [ds,dp,ddelta,dpsi]=ddEllipsometry(layerInterface,substratePermittivity,lambda0,theta,parameters{iP1},parameters{iP2});
    [dsJCM,dpJCM,ddeltaJCM,dpsiJCM]=ddEllipsometryJCM(t,iK,parameters{iP1},parameters{iP2});
    
    [ds dp;dsJCM dpJCM];
    
    fprintf("\nAccuracy of second derivative w.r.t. \'%s\', \'%s\':\n",parameters{iP1},parameters{iP2});
    fprintf('jonesSS:   %e  (reference = %s)\n',abs(ds-dsJCM),formatComplex(ds));
    fprintf('jonesPP:   %e  (reference = %s)\n',abs(dp-dpJCM),formatComplex(dp));
  end
end

%end


function [ds,dp,ddelta,dpsi]=EllipsometryJCM(t,iK)
dp=t.Jones_ps{1}(iK,1,1);
ds=t.Jones_ps{1}(iK,2,2);
ddelta=t.Delta_ps;
dpsi=t.Psi_ps;
end

function [ds,dp,ddelta,dpsi]=dEllipsometryJCM(t,iK,dPara_)
dPara = sprintf('d_%s',dPara_);
dp = t.(dPara).Jones_ps{1}(iK,1,1);
ds = t.(dPara).Jones_ps{1}(iK,2,2);
ddelta = t.(dPara).Delta_ps;
dpsi   = t.(dPara).Psi_ps;
end

function [ds,dp,ddelta,dpsi]=ddEllipsometryJCM(t,iK,dPara1_,dPara2_)
dPara1 = sprintf('d_%s',dPara1_);
dPara2 = sprintf('d_%s',dPara2_);
if (~isfield(t.(dPara1),dPara2))
    temp = dPara1;
    dPara1 = dPara2;
    dPara2 = temp;
end
dp = t.(dPara1).(dPara2).Jones_ps{1}(iK,1,1);
ds = t.(dPara1).(dPara2).Jones_ps{1}(iK,2,2);
ddelta = t.(dPara1).(dPara2).Delta_ps;
dpsi   = t.(dPara1).(dPara2).Psi_ps;
end

function [ds,dp,ddelta,dpsi]=dEllipsometry(layerInterface,substratePermittivity,lambda0,theta,dPara)
layerInterface_=autodiff(layerInterface,0);
substratePermittivity_=autodiff(substratePermittivity,0);
lambda0_=autodiff(lambda0,0);
theta_=autodiff(theta,0);

if strcmp(dPara,"LayerInterface") layerInterface_=autodiff(layerInterface,1e-9);end
if strcmp(dPara,"SubstratePermittivity") substratePermittivity_=autodiff(substratePermittivity,1);end
if strcmp(dPara,"Lambda0") 
  substratePermittivity_=autodiff(substratePermittivity,1e-3);
  lambda0_=autodiff(lambda0,1e-9);
end
if strcmp(dPara,"Theta") theta_=autodiff(theta,1);end
[s,p,delta,psi]=Ellipsometry(layerInterface_,substratePermittivity_,lambda0_,theta_);

ds=value(s,1);
dp=value(p,1);
ddelta=value(delta,1);
dpsi=value(psi,1);

end


function [dds,ddp,dddelta,ddpsi]=ddEllipsometry(layerInterface,substratePermittivity,lambda0,theta,dPara1,dPara2)
layerInterface_=autodiff(layerInterface,0);
substratePermittivity_=autodiff(substratePermittivity,0);
lambda0_=autodiff(lambda0,0);
theta_=autodiff(theta,0);

if strcmp(dPara1,"LayerInterface") layerInterface_=autodiff(layerInterface,1e-9);end
if strcmp(dPara1,"SubstratePermittivity") substratePermittivity_=autodiff(substratePermittivity,1);end
if strcmp(dPara1,"Lambda0") 
  substratePermittivity_=autodiff(substratePermittivity,1e-3);
  lambda0_=autodiff(lambda0,1e-9);
end
if strcmp(dPara1,"Theta") theta_=autodiff(theta,1);end

layerInterface__=autodiff(layerInterface_,0);
substratePermittivity__=autodiff(substratePermittivity_,0);
lambda0__=autodiff(lambda0_,0);
theta__=autodiff(theta_,0);

if strcmp(dPara2,"LayerInterface") layerInterface__=autodiff(layerInterface_,1e-9);end
if strcmp(dPara2,"SubstratePermittivity") substratePermittivity__=autodiff(substratePermittivity_,1);end
if strcmp(dPara2,"Lambda0") 
  substratePermittivity__=autodiff(substratePermittivity_,1e-3);
  lambda0__=autodiff(lambda0_,1e-9);
end
if strcmp(dPara2,"Theta") theta__=autodiff(theta_,1);end

[s,p,delta,psi]=Ellipsometry(layerInterface__,substratePermittivity__,lambda0__,theta__);
dds=value(value(s,1),1);
ddp=value(value(p,1),1);
dddelta=value(value(delta,1),1);
ddpsi=value(value(psi,1),1);

end




function [s,p,delta,psi]=Ellipsometry(layerInterface,substratePermittivity,lambda0,theta)
n1=1.0;
n2=sqrt(substratePermittivity);
theta=theta/180*pi;
beta=asin(sin(theta)*n1/n2);

s=sin(theta-beta)/sin(theta+beta);
p=tan(theta-beta)/tan(theta+beta);

kz=cos(theta)*2*pi/lambda0;
s=s*exp(-2i*kz*layerInterface);
p=p*exp(-2i*kz*layerInterface);
[delta,psi]=deltapsi(s,p);
end



function [delta,psi]=deltapsi(s,p)
rho=p/s;
delta=log(rho);
psi=atan(abs(rho));
end


function cs=formatComplex(c)
if (imag(c)==0.0)
  cs=sprintf('%e',c);
elseif (imag(c)>0)
    cs=sprintf('%e + %ei',real(c),imag(c));
else
    cs=sprintf('%e - %ei',real(c),-imag(c));  
end
end
    