%% 3D quantum dot emitter 
%% This driver performs a scan over wavelength of the dipole

%% vacuum wavelengths
lambdas = [965 : 0.1 : 975]*1e-9;

emissions = [];
emissions_background = [];
total_radiations = [];

for iRun = 1 : length(lambdas)
  keys.lambda0 = lambdas(iRun);

  %% solve the project 
  %% use temporary project folder (option 'temporary'='yes')
  results = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');

  %% gather results
  
  emission=results{2}.DipoleEmission;
  emission_background=results{2}.DipoleBulkEmission;
  radiation=results{4}.Radiation;

  emissions(iRun, :) = emission.';
  emissions_background(iRun, :) = emission_background.';

  for iPol=1 : 3
    total_radiations(iRun, iPol) = sum(radiation{iPol});
  end    
end
  
%% plot the results
fig=figure();
set(fig, 'Units', 'normalized', 'Position', [0.2,0.1,0.8,0.4]);
xyz = 'xyz';
pol_color = 'rgb'; % line color for different dipole-polarizations
pol_icon = 'ox+'; % line symbolr for different dipole-polarizations 

%% plot efficiencies
subplot(1, 2, 1)
efficiencies = total_radiations./emissions;
legend_entries = {};
for iPol=1 : 3
  plot(lambdas/1e-9, efficiencies(:, iPol), sprintf('%c%c--', pol_color(iPol), pol_icon(iPol)), 'LineWidth', 2);
  hold on
  legend_entries{end+1} = sprintf('efficiency (%c-pol)', xyz(iPol));
end
legend(legend_entries, 'Location', 'best');
xlabel('\lambda [nm]');
title('Efficiency vs. wavelength');
set(gca, 'FontSize', 12);

%% plot purcel factors
%% the Purcell factor is the quotient between the dipole emission in the cavity
%% and the emission of a dipole in a homogeneous space with the background material
%% at the dipole position
subplot(1, 2, 2)
purcell_factors = emissions./emission_background;
legend_entries = {};
for iPol=1 : 3
  plot(lambdas/1e-9, purcell_factors(:, iPol), sprintf('%c%c-', pol_color(iPol), pol_icon(iPol)), 'LineWidth', 2);
  hold on
  legend_entries{end+1} = sprintf('Purcell factor (%c-pol)', xyz(iPol));
end
xlabel('\lambda [nm]');
legend(legend_entries, 'Location', 'best');
title('Purcell factor vs. wavelength');
set(gca, 'FontSize', 12);

