% example : rotationally symmetric VCSEL, cold cavity
% compare setup and results from Bienstman et al.
% IEEE JOURNAL OF QUANTUM ELECTRONICS, VOL. 37, 1618 (2001)

jcm_root = getenv('JCMROOT'); addpath(fullfile(jcm_root, 'ThirdPartySupport', 'Matlab'));
jcmwave_set_num_threads(1);

% solver
c0 = 299792458;
lambdaGuess = 980.6e-9;
keys.omegaGuess = round(c0*2*pi/lambdaGuess*1e-15, 3)*1e15;
keys.n_modes = 1;
keys.n_refinement_steps = 0;
keys.fem_degree = 4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute VCSEL resonance mode

these_results = jcmwave_solve('project.jcmp', keys, 'workingdir', 'my_working_dir');

ev = these_results{1}.eigenvalues.eigenmode(1); 
res_wavelength = c0*2*pi./real(ev);
q_factor = real(ev)/(-2*imag(ev));
n_unknowns = these_results{1}.computational_costs.Unknowns(end);
cpu_time = sum(these_results{1}.computational_costs.CpuTime);

results = [];
results(1) = res_wavelength;
results(2) = q_factor;
results(3) = keys.fem_degree;
results(4) = keys.n_refinement_steps;
results(5) = n_unknowns;
results(6) = cpu_time;
save_name = ['results_vcsel.txt'];
save('-ascii', save_name, 'results');
fprintf('L = %5.5f nm, Q =  %5.5f \n', res_wavelength*1e9, q_factor);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Post-process: export fields

pp_result = jcmwave_solve('postprocess_export_fields.jcmp', keys, 'workingdir', 'my_working_dir');
cfb = pp_result{1};

data = sum(abs(cfb.field{1}).^2, 3); data = data/max(max(data));
figure(1)
  surf(cfb.X*1e9, cfb.Y*1e9, data); 
  shading interp; view(2); axis equal;colorbar;


