% example : rotationally symmetric VCSEL, cold cavity
% compare setup and results from Bienstman et al.
% IEEE JOURNAL OF QUANTUM ELECTRONICS, VOL. 37, 1618 (2001)
clear keys;
jcm_root = getenv('JCMROOT');
addpath(fullfile(jcm_root, 'ThirdPartySupport', 'Matlab'));
jcmwave_set_num_threads(1);

keys.uol = 1e-9;
%Bragg mirrors
keys.n_top_dbr = 25; 
keys.n_bot_dbr = 30; 
% Aperture diameter
keys.diameter = 6000;
% Aperture position
keys.oxide_window_x = 47.78;
%thicknesses 
keys.t_dbr_gaas = 69.49;
keys.t_dbr_algaas = 79.63;
keys.t_ox = 15.93;
keys.t_cav_gaas = 136.49;
keys.t_cav_qw = 5;

% materials
keys.n_Air = 1.0;
keys.n_GaAs = 3.53;
keys.n_AlGaAs = 3.08;
keys.n_AlAs = 2.95;
keys.n_AlOx = 1.6;
keys.n_QW_act = 3.53; % (cold cavity) 
keys.n_QW_pass = 3.53 + 0.01i;

% computational domain
keys.offset_x = 1500;
keys.offset_y = 100;
keys.slc = 400; % meshing sidelength constraint

% solver
c0 = 299792458;
lambdaGuess = 980.6e-9;
keys.omegaGuess = round(c0*2*pi/lambdaGuess*1e-15, 3)*1e15;
keys.n_modes = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% check convergence of the results by varying number of refinement steps
% and by varying finite element degree

steps = [0];   % [0 1 2];
degs = [4];      % [3 4 5];
counter = 0; results = [];

for iiii = 1:length(degs)
for iii = 1:length(steps)
  counter = counter + 1;
  keys.n_refinement_steps = steps(iii);
  keys.fem_degree = degs(iiii);

  these_results = jcmwave_solve('project.jcmp', keys);

  ev = these_results{1}.eigenvalues.eigenmode(1); 
  res_wavelength = c0*2*pi./real(ev);
  q_factor = real(ev)/(-2*imag(ev));
  n_unknowns = these_results{1}.computational_costs.Unknowns(end);
  cpu_time = sum(these_results{1}.computational_costs.CpuTime);

  results(counter, 1) = res_wavelength;
  results(counter, 2) = q_factor;
  results(counter, 3) = keys.fem_degree;
  results(counter, 4) = keys.n_refinement_steps;
  results(counter, 5) = n_unknowns;
  results(counter, 6) = cpu_time;
  save_name = ['results_vcsel.txt'];
  save('-ascii', save_name, 'results');
 
  fprintf('L = %5.5f nm, Q =  %5.5f \n', res_wavelength*1e9, q_factor);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
