jcm_root = getenv('JCMROOT');
addpath(fullfile(jcm_root, 'ThirdPartySupport', 'Matlab'));

% Dielectric Meta Surface
% This simple driver the simulation
%
% This example is taken from
% Berzins, Jonas, Fabrizio Silvestri, Giampiero Gerini, Frank Setzpfandt, Thomas Pertsch, and Stefan MB Baeumer.
% "Color filter arrays based on dielectric metasurface elements." 
% In Metamaterials XI, vol. 10671, p. 106711F. International Society for Optics and Photonics, 2018.

%%define default key values
keys.pitch = 250.0;


results = jcmwave_solve('project.jcmp');

%% process the computed data to extract reflection and transmission
uol=1e-9;
c0   = 299792458;
mu0  = 4e-7*pi;
eps0 = 1./(mu0*c0^2);
Z0 = sqrt(mu0/eps0);
area = (keys.pitch*uol)^2;
pIn = 1.0*area;

% to compute reflection and transmission coeffcients, the power flux densities 
% in both directions are computed from the corresponding FourierTransforms
pf = jcmwave_convert2powerflux(results{2});
R_p = sum(pf.PowerFluxDensity{1}(:,3));
R_s = sum(pf.PowerFluxDensity{2}(:,3));

pf = jcmwave_convert2powerflux(results{3});
T_p = -sum(pf.PowerFluxDensity{1}(:,3));
T_s = -sum(pf.PowerFluxDensity{2}(:,3));

absorption = results{6}.ElectromagneticFieldAbsorption;
A_p = real(absorption{1}(3))/pIn;
A_s = real(absorption{2}(3))/pIn;

J = squeeze(results{5}.Jones_ps{1});
t_p = abs(J(1,1));
phase_p = angle(J(1,1));

%%% 
fprintf('\nTotal transmission for p-polarized incoming field: T=%g%%',R_p*100.0);
fprintf('\nTotal reflection for p-polarized incoming field:   R=%g%%',T_p*100.0);
fprintf('\nTotal absorption for p-polarized incoming field:   A=%g%%',A_p*100.0);
      
fprintf('\nTotal transmission for s-polarized incoming field: T=%g%%',R_p*100.0);
fprintf('\nTotal reflection for s-polarized incoming field:   R=%g%%',T_p*100.0);
fprintf('\nTotal absorption for s-polarized incoming field:   A=%g%%',A_p*100.0);
      
fprintf('\nThe transmitted light undergoes a phase shift of %.2g pi',phase_p/pi);

