clear all;

mu = linspace(1.0, 4.5, 10);

keys = {};
for mm = 1:length(mu)
	keys.mu = mu(mm);
	results = jcmwave_solve('project.jcmp', keys);

	Sigma_ext(mm) = real(results{2}.ExtinctionElectromagneticChiralityFlux{1});
	Sigma_sca(mm) = real(results{3}.ElectromagneticChiralityFlux{1});
	Sigma_conv(mm) = real(results{4}.ElectromagneticChiralityConversionFlux{1});
end

figure(); hold on; grid on;
plot(mu, -Sigma_ext, 'bx-.');
plot(mu, Sigma_sca, 'kx-.');
plot(mu, Sigma_conv, 'rx-.');
legend({'-\Sigma_{ext}', '\Sigma_{sca}', '\Sigma_{conv}'}, 'Location', 'NorthWest');
xlabel('\mu');
ylabel('a.u.');

chi = 2.0*real(results{5}.field{1});
X = results{5}.X;
Y = results{5}.Y;
chi(sqrt(X.^2+Y.^2) > 1.0) = nan();
chi_0 = pi*8.85418781762e-12;

figure();
surf(X, Y, chi/chi_0);
view(2); colormap jet; shading interp;
axis equal; axis off;
colorbar; caxis([0.0 1.8]);

