import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import numpy as np
import matplotlib.pyplot as plt
import jcmwave

# Optical chirality of a quarter-wave plate
# This simple driver directly starts the simulation
#
# This examples is taken from
# Philipp Gutsche, Lisa V. Poulikakos, Martin Hammerschmidt, Sven Burger, and Frank Schmidt. 
# Time-harmonic optical chirality in inhomogeneous space. 
# In SPIE OPTO, pages 97560X-97560X. International Society for
# Optics and Photonics, 2016.

results= jcmwave.solve('project.jcmp')

chi = 2.0*np.real(results[4]['field'][0][:,:,0])
X = results[4]['X']
Y = results[4]['Y']
chi[np.where(np.sqrt(X**2+Y**2) > 1.0)] = np.nan
chi_0 = np.pi*8.85418781762e-12

plt.pcolormesh(X,Y,chi/chi_0, shading='gouraud', vmin=0.0, vmax=1.8)
plt.colorbar()
plt.axis('equal')
plt.axis('off')
plt.show()
