%% Optical chirality of an isolated scatterer
% This simple driver directly starts the simulation
%
% This examples is taken from
% Philipp Gutsche, Lisa V. Poulikakos, Martin Hammerschmidt, Sven Burger, and Frank Schmidt. 
% Time-harmonic optical chirality in inhomogeneous space. 
% In SPIE OPTO, pages 97560X-97560X. International Society for Optics and Photonics, 2016.

close all;

keys = {};
keys.lambda0 = 1.0;
keys.k = 2.*pi/keys.lambda0;
keys.omega = keys.k * 299792458.0;

n = [1.5, 1.5, 1.7];
keys.eps = diag(n.^2.);

keys.t = .25*keys.lambda0/(n(3)-n(1));
keys.t_cd = keys.lambda0;
keys.w = 2.5*keys.lambda0;

keys.slc = .1*keys.lambda0;
keys.slc2 = keys.slc / max(n);

results = jcmwave_solve('project.jcmp', keys, 'jcmt_pattern', 'matlab');

% vacuum wave impedance
Z0 = 3.767303134617790e+02;
% incident energy flux
S0 = .5/Z0*keys.w;
% chirality flux of circularly polarized plane wave
Sigma0 = keys.k*S0;

Sigma_ref = real(results{2}.ElectromagneticChiralityFlux{1}(results{2}.DomainIdFirst==1)) / Sigma0;
Sigma_trans = real(results{2}.ElectromagneticChiralityFlux{1}(results{2}.DomainIdFirst==3)) / Sigma0;

Sigma_conv_ref = real(results{3}.ElectromagneticChiralityConversionFlux{1}(results{3}.DomainIdFirst==1)) / Sigma0;
Sigma_conv_trans = real(results{3}.ElectromagneticChiralityConversionFlux{1}(results{3}.DomainIdFirst==2)) / Sigma0;

chi_conv = -2.*keys.omega*imag(results{4}.ElectricChirality{1}) / Sigma0;

fprintf('\n\n');
fprintf('Sigma_ref = %g\n', Sigma_ref);
fprintf('Sigma_trans = %g\n', Sigma_trans);
fprintf('Sigma_conv_ref = %g\n', Sigma_conv_ref);
fprintf('Sigma_conv_trans = %g\n', Sigma_conv_trans);
fprintf('chi_conv = %g\n', chi_conv);

fprintf('\n0 = Sigma_inc = Sigma_ref + Sigma_trans + Sigma_conv + chi_conv = %g\n', ...
	Sigma_ref + Sigma_trans + Sigma_conv_ref + Sigma_conv_trans + chi_conv);

% vacuum permittivity
eps0 = 8.85418781762e-12;
% energy density of plane wave
U0 = .5*eps0;
% chirality density of circularly polarized plane wave
chi0 = keys.k*U0;

figure(); hold on;
surf(results{5}.X, results{5}.Y, imag(results{5}.field{1})/chi0);
line(keys.w*[-.6,.6], -keys.t*[.5,.5], [5.,5.], 'LineWidth', 2, 'Color', 'k');
line(keys.w*[-.6,.6], keys.t*[.5,.5], [5.,5.], 'LineWidth', 2, 'Color', 'k');
view(2); shading interp; colormap jet; axis equal; colorbar; %axis off;
ylim([min(results{5}.Y(:)), max(results{5}.Y(:))]);

figure(); hold on;
surf(results{5}.X, results{5}.Y, real(results{5}.field{1} + results{6}.field{1})/chi0);
line(keys.w*[-.6,.6], -keys.t*[.5,.5], [5.,5.], 'LineWidth', 2, 'Color', 'k');
line(keys.w*[-.6,.6], keys.t*[.5,.5], [5.,5.], 'LineWidth', 2, 'Color', 'k');
view(2); shading interp; colormap jet; axis equal; colorbar; axis off;
ylim([min(results{5}.Y(:)), max(results{5}.Y(:))]);
