import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import numpy as np
import matplotlib.pyplot as plt
import jcmwave

## Optical chirality of an isolated scatterer
# This simple driver directly starts the simulation
#
# This examples is taken from
# Philipp Gutsche, Lisa V. Poulikakos, Martin Hammerschmidt, Sven Burger, and Frank Schmidt. 
# Time-harmonic optical chirality in inhomogeneous space. 
# In SPIE OPTO, pages 97560X-97560X. International Society for Optics and Photonics, 2016.

keys = dict()
keys['lambda0'] = 1.0
keys['k'] = 2*np.pi/keys['lambda0']
keys['omega'] = keys['k'] * 299792458.0

n = np.array([1.5, 1.5, 1.7])
keys['eps'] = np.diag(n**2.)

keys['t'] = .25*keys['lambda0']/(n[2]-n[0])
keys['t_cd'] = keys['lambda0']
keys['w'] = 2.5*keys['lambda0']

keys['slc'] = .1*keys['lambda0']
keys['slc2'] = keys['slc'] / max(n)

results = jcmwave.solve('project.jcmp', keys, jcmt_pattern='python')

# vacuum wave impedance
Z0 = 3.767303134617790e+02
# incident energy flux
S0 = .5/Z0*keys['w']
# chirality flux of circularly polarized plane wave
Sigma0 = keys['k']*S0

Sigma_ref = np.real(results[1]['ElectromagneticChiralityFlux'][0][np.where(results[2-1]['DomainIdFirst']==1)][0]) / Sigma0
Sigma_trans = np.real(results[1]['ElectromagneticChiralityFlux'][0][np.where(results[2-1]['DomainIdFirst']==3)][0]) / Sigma0

Sigma_conv_ref = np.real(results[2]['ElectromagneticChiralityConversionFlux'][0][np.where(results[3-1]['DomainIdFirst']==1)][0]) / Sigma0
Sigma_conv_trans = np.real(results[2]['ElectromagneticChiralityConversionFlux'][0][np.where(results[3-1]['DomainIdFirst']==2)][0]) / Sigma0

chi_conv = -2*keys['omega']*np.imag(results[3]['AnisotropicElectricChirality'][0][0]) / Sigma0

print('Sigma_ref = {:g}'.format( Sigma_ref))
print('Sigma_trans = {:g}'.format( Sigma_trans))
print('Sigma_conv_ref = {:g}'.format( Sigma_conv_ref))
print('Sigma_conv_trans = {:g}'.format( Sigma_conv_trans))
print('chi_conv = {:g}'.format( chi_conv))

print('0 = Sigma_inc = Sigma_ref + Sigma_trans + Sigma_conv + chi_conv = {:g}'.format( 
	Sigma_ref + Sigma_trans + Sigma_conv_ref + Sigma_conv_trans + chi_conv))

# vacuum permittivity
eps0 = 8.85418781762e-12
# energy density of plane wave
U0 = .5*eps0
# chirality density of circularly polarized plane wave
chi0 = keys['k']*U0

plt.pcolormesh(results[4]['X'], results[4]['Y'], np.imag(results[4]['field'][0][:,:,0])/chi0,
               cmap='jet', shading='gouraud')
plt.plot(keys['w']*np.array([-.6,.6]), -keys['t']*np.array([.5,.5]), lineWidth=2, color='k')
plt.plot(keys['w']*np.array([-.6,.6]), keys['t']*np.array([.5,.5]), lineWidth=2, color='k')
plt.axis('equal')
plt.axis('off')
plt.colorbar()
plt.show()

plt.pcolormesh(results[4]['X'], results[4]['Y'],
               np.real(results[4]['field'][0] + results[6-1]['field'][0])[:,:,0]/chi0,
               cmap='jet', shading='gouraud')
plt.plot(keys['w']*np.array([-.6,.6]), -keys['t']*np.array([.5,.5]), lineWidth=2, color='k')
plt.plot(keys['w']*np.array([-.6,.6]), keys['t']*np.array([.5,.5]), lineWidth=2, color='k')
plt.axis('equal')
plt.axis('off')
plt.colorbar()
plt.show()
