% Driver for the calculation of Propagating Modes of hollow-core photonic crystal fiber

clear keys;

% project parameters
keys.refractive_index = 1.4585;
keys.vacuum_wavelength = 589.0e-9;

keys.a_nm = 1503; % periodicity
keys.hexa_pore_strut_width_nm = 43.2;
keys.hexa_pore_corner_rounding_nm = 285;
keys.hexa_pore_corner_rounding_n = 4;
keys.central_pore_corner_rounding_nm = 1380;
keys.central_pore_corner_rounding_n = 6;
keys.central_pore_size = 2; %(number of omitted hexagonal rings)
keys.d_inner_cladding_ring_nm = 172;
keys.n_cladding_rings = 2; 
keys.d_outer_ring_nm = 1000;  % -300 

% numerical parameters: different regions in coarse mesh 
keys.slc_background = 1500; % 500
keys.slc_core = 800;
keys.slc_pore = 600;

% FEM parameters
keys.fem_degree = 3;
keys.n_eigenvalues = 2;
keys.n_refinements = 0;
keys.info_level = 3;

keys.eigenvalue_guess = 9.94e-01;
keys.precision = 1e-5;

% start computation: (1) .jcm input files are written, (2) mesh is created (grid.jcm), (3) FEM project is solved:
results = jcmwave_solve('project.jcmp', keys);

% display some results:
fprintf('\nNumber of unknowns in FEM computation: %i', results{1}.computational_costs.Unknowns(end));
n_modes = length(results{1}.eigenvalues.effective_refractive_index);
for ii = 1:n_modes
  n_eff_real = real(results{1}.eigenvalues.effective_refractive_index(ii));
  n_eff_imag = imag(results{1}.eigenvalues.effective_refractive_index(ii));
  fprintf('\nMode %i, Re(n)=%8.8f, Im(n)=%8.8f', ii, n_eff_real, n_eff_imag);
end


