import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import numpy as np
import jcmwave

## 2D simple solar cell example
## This simple driver directly starts the simulation
results = jcmwave.solve('project.jcmp')

# incoming flux for normalization
# constants
c0   = 299792458
mu0  = 4*np.pi*1e-7
eps0 = 1/(mu0*c0**2)
Z0   = np.sqrt(mu0/eps0)
pitch = results[2]['header']['BBox_x_max'] - results[2]['header']['BBox_x_min']
p_in = 0.5/Z0*pitch* np.sqrt(results[2]['header']['RelPermittivityIncoming'])

## Absorbed energy
absorption = results[1]['ElectromagneticFieldAbsorption'][0]/p_in

# Fourier Transform
e_field = results[2]['ElectricFieldStrength'][0]
K = np.array(results[2]['K'])                             
km = np.linalg.norm(K[np.ceil(len(K)/2)-1,:])
angles = np.real(np.arccos(-K[:,1]/km))
refl = np.sum(np.conj(e_field)*e_field,axis=1).dot(np.cos(angles))
reflection = np.sum(refl,axis=0)*0.5/Z0*np.sqrt(results[2]['header']['RelPermittivity'])*pitch/p_in

## Flux outgoing
energy_flux = results[3]['ElectromagneticFieldEnergyFlux'][0]
index = np.logical_and(results[3]['DomainIdFirst'] == 2, results[3]['DomainIdSecond'] == 3)
absorption_via_flux = np.real(energy_flux[index])/p_in

index = np.logical_and(results[3]['DomainIdFirst'] == 2, results[3]['DomainIdSecond'] == 1)
reflection_via_flux = np.real(p_in+energy_flux[index])/p_in;
