% Project: benchmark to published results
% Besbes et al. JEOS Rap. Comm. 2, 07022 (2007)
% Numerical analysis of a slit-groove diffraction problem
% See also: Proc. SPIE Vol. 8880 (2013) 88801Z, arxiv.org/pdf/1310.2732
  
jcmwave_startup();
jcmwave_set_num_threads(1);

% geometry
keys.uol = 1e-9;
keys.d = 500; % slit groove distance
keys.t_ag = 400;
keys.t_groove = 100;
keys.w_groove = 100;
keys.w_slit = 100;
keys.depth_detector = 400;
keys.w_detector = 200;
keys.offset = 150; % computational domain offset

% refractive indices
keys.n_1 = 1; % air
keys.n_2 = 1.5; % SiO2
keys.n_3 = sqrt(-33.22 + 1.17i);

% source
keys.vacuum_wavelength = 852.0e-9;

% numerical parameters:
keys.n_refinement_steps = 0;
keys.fem_degree = 3;
keys.precision = 1e-3;
keys.delta_x_corner = 0.1;
keys.mesh_sidelength = 50;
keys.info_level = -1;

%%%%%%%%%%%%%%%%%%%%%%%%%% WITHOUT GROOVE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
keys.with_groove = false;
run2d = jcmwave_solve('project.jcmp', keys);
s0 = real(run2d{2}.ElectromagneticFieldEnergyFlux{1});
n_unknowns1 = run2d{1}.computational_costs.Unknowns(end);
copyfile('layout.jcm', 'layout_without_groove.jcm');
copyfile('grid.jcm', 'grid_without_groove.jcm');
%%%%%%%%%%%%%%%%%%%%%%%%%% WITH GROOVE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
keys.with_groove = true;
run2d = jcmwave_solve('project.jcmp', keys);
s1 = real(run2d{2}.ElectromagneticFieldEnergyFlux{1});
n_unknowns2 = run2d{1}.computational_costs.Unknowns(end);
copyfile('layout.jcm', 'layout_with_groove.jcm');
copyfile('grid.jcm', 'grid_with_groove.jcm');
%%%%%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

s1_s0 = s1/s0;
n_unknowns = n_unknowns1 + n_unknowns2;

fprintf('\nS1/S0 = %8.8f, N = %i\n', s1_s0, n_unknowns);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

