%% PhC cavity
%% This simple driver directly starts the simulation


results = jcmwave_solve('project.jcmp');

ev=results{1}.eigenvalues;
resonance_frequency = real(ev.eigenmode(1));
c0 = 299792458; 
resonance_wavelength = c0*2*pi/resonance_frequency;
quality_factor = resonance_frequency/(-2*imag(ev.eigenmode(1)));

fprintf('\nQuality factor: %2.4g, @ lambda=%2.4g\n', quality_factor, resonance_wavelength);

%% visualize exported cross-section through near field (z=0)
fieldbag_cartesian_xy = results{2};
intensity_xy = sum(abs(fieldbag_cartesian_xy.field{1}).^2, 3);
intensity_scale=max(max(intensity_xy));
intensity_xy = intensity_xy/intensity_scale;

figure()
surf(fieldbag_cartesian_xy.X*1e6, fieldbag_cartesian_xy.Y*1e6, intensity_xy);
shading interp; view(2); axis equal; colorbar;
title('Intensity (xy-plane)')
xlabel('X [microns]'); ylabel('Y [microns]');

%% visualize exported cross-section through near field (y=0)
fieldbag_cartesian_xz = results{3};
intensity_xz = sum(abs(fieldbag_cartesian_xz.field{1}).^2, 3);
intensity_xz = intensity_xz/intensity_scale;

figure()
surf(fieldbag_cartesian_xz.X*1e6, fieldbag_cartesian_xz.Z*1e6, intensity_xz);
shading interp;  view(2); axis equal; colorbar;
title('Intensity (xz-plane)')
xlabel('X [microns]'); ylabel('Z [microns]');
