import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import numpy as np
import matplotlib.pyplot as plt
import jcmwave

## PhC cavity
## This simple driver directly starts the simulation

results = jcmwave.solve('project.jcmp')

ev=results[0]['eigenvalues']
resonance_frequency = np.real(ev['eigenmode'][0])
c0 = 299792458
resonance_wavelength = c0*2*np.pi/resonance_frequency
quality_factor = resonance_frequency/(-2*np.imag(ev['eigenmode'][0]));

print('Quality factor: {:2.4g}, @ lambda={:2.4g}'.format(quality_factor, resonance_wavelength))

## visualize exported cross-section through near field (z=0)
fieldbag_cartesian_xy = results[1]
intensity_xy = np.sum(np.abs(fieldbag_cartesian_xy['field'][0])**2, axis=2)
intensity_scale = np.max(np.max(intensity_xy))
intensity_xy = intensity_xy/intensity_scale

plt.pcolormesh(fieldbag_cartesian_xy['X']*1e6, fieldbag_cartesian_xy['Y']*1e6, intensity_xy, shading='gouraud')
plt.colorbar()
plt.axis('equal')
plt.title('Intensity (xy-plane)')
plt.xlabel('X [microns]')
plt.ylabel('Y [microns]')
plt.show()

## visualize exported cross-section through near field (y=0)
fieldbag_cartesian_xz = results[2];
intensity_xz = np.sum(np.abs(fieldbag_cartesian_xz['field'][0])**2, axis=2)
intensity_xz = intensity_xz/intensity_scale

plt.pcolormesh(fieldbag_cartesian_xz['X']*1e6, fieldbag_cartesian_xz['Z']*1e6, intensity_xz, shading='gouraud')
plt.colorbar()
plt.axis('equal')
plt.title('Intensity (xz-plane)')
plt.xlabel('X [microns]')
plt.ylabel('Z [microns]')
plt.show()

