%% 3D quantum dot emitter with micro lens (exploiting rotational symmetry)
%% The extracted light is coupled into a glass fiber
%% This simple driver directly starts the simulation


%% solve the problems in order: the fiber mode problem is solved first
all_results = jcmwave_solve({'fiber_modes/project.jcmp', './project.jcmp'});

% reshape the results as all_results is as a cell array of cell arrays 
results = [all_results{:}]; 
% results{1} is the output of the fiber mode problem, results{2-4} are the
% outputs of project.jcmp

emission=results{3}.DipoleEmission(1);
mode_coefficients=results{4}.ModeCoefficients{1};

%% each fiber mode is scaled ot unit energy flux through cross section
%% summing up over the squares of the mode expansion coefficients gives
%% the total energy flux within the fiber. 
mode_coupling=sum(abs(mode_coefficients.^2));
 
%% the mode coupling efficiency is the quotient between the dipole emission
%% and the energy in the waveguide modes

fprintf('\n\n*** Mode coupling efficiency: %2.4g%%\n', mode_coupling/emission*100);
