import sys
import os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass

import numpy as np
import jcmwave

## 3D quantum dot emitter with micro lens (exploiting rotational symmetry)
## The extracted light is coupled into a glass fiber
## This simple driver directly starts the simulation
results = jcmwave.solve(['fiber_modes/project.jcmp', './project.jcmp'])

emission=results[1][1]['DipoleEmission'][0]
mode_coefficients=results[1][2]['ModeCoefficients'][0]

## each fiber mode is scaled ot unit energy flux through cross section
## summing up over the squares of the mode expansion coefficients gives
## the total energy flux within the fiber. 
mode_coupling = np.sum(np.abs(mode_coefficients**2))
 
## the mode coupling efficiency is the quotient between the dipole emission
## and the energy in the waveguide modes

print('\n\n*** Mode coupling efficiency: {:2.4g}%'.format(mode_coupling/emission*100))
