%% 3D quantum dot emitter (exploiting rotational symmetry)
%% This driver performs a scan over wavelength of the dipole

%% vacuum wavelengths
lambdas=[0.7 : 0.01 : 1.2]*1e-6;

scaled_emissions = [];
scaled_absorptions = [];

for iRun = 1 : length(lambdas)
  keys.lambda0 = lambdas(iRun);

  %% solve the project 
  %% use temporary project folder (option 'temporary'='yes')
  results = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');

  %% gather results
  
  emission_background=results{2}.DipoleBulkEmission;
  scaled_emissions(end+1)=results{2}.DipoleEmission/emission_background;
  scaled_absorptions(end+1)=sum(results{3}.ElectromagneticFieldAbsorption{1})/emission_background;
end
  
%% plot the results
fig=figure();
plot(lambdas*1e6, [scaled_emissions; scaled_absorptions], 'LineWidth', 2);
legend('Radiative', 'Absorp.');
xlabel('lambda [\mu]');
ylabel('Normalized decay rates');
set(gca, 'FontSize', 12);
